@@ -1,6 +1,6 @@
 /*
  * coding.c - data encoding functions
- * Copyright (C) 2012-2016 Simon Josefsson
+ * Copyright (C) 2012-2020 Simon Josefsson
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -25,6 +25,8 @@
 
 #include <stdlib.h>
 
+#include "attribute.h"		/* FALLTHROUGH */
+
 #include "base32.h"
 #include "c-ctype.h"
 
@@ -99,7 +101,7 @@ hex_decode (char hex)
  *   error code is returned.
  **/
 int
-oath_hex2bin (const char *hexstr, char *binstr, size_t * binlen)
+oath_hex2bin (const char *hexstr, char *binstr, size_t *binlen)
 {
   bool highbits = true;
   size_t save_binlen = *binlen;
@@ -203,7 +205,7 @@ oath_bin2hex (const char *binstr, size_t binlen, char *hexstr)
  * Since: 1.12.0
  **/
 int
-oath_base32_decode (const char *in, size_t inlen, char **out, size_t * outlen)
+oath_base32_decode (const char *in, size_t inlen, char **out, size_t *outlen)
 {
   size_t i, j, tmplen = 0;
   char *in_upcase;
@@ -226,13 +228,17 @@ oath_base32_decode (const char *in, size_t inlen, char **out, size_t * outlen)
     case 2:
       in_upcase[j++] = '=';
       in_upcase[j++] = '=';
+      FALLTHROUGH;
     case 4:
       in_upcase[j++] = '=';
+      FALLTHROUGH;
     case 5:
       in_upcase[j++] = '=';
       in_upcase[j++] = '=';
+      FALLTHROUGH;
     case 7:
       in_upcase[j++] = '=';
+      FALLTHROUGH;
     default:
     case 0:
       break;
@@ -289,7 +295,7 @@ oath_base32_decode (const char *in, size_t inlen, char **out, size_t * outlen)
  * Since: 1.12.0
  **/
 int
-oath_base32_encode (const char *in, size_t inlen, char **out, size_t * outlen)
+oath_base32_encode (const char *in, size_t inlen, char **out, size_t *outlen)
 {
   char *tmp;
   size_t len = base32_encode_alloc (in, inlen, &tmp);
