@@ -64,6 +64,18 @@ main (void)
       return 1;
     }
 
+  if (!pskc_check_version ("0"))
+    {
+      printf ("pskc_check_version (0) didn't fail?!\n");
+      return 1;
+    }
+
+  if (PSKC_VERSION_NUMBER < 0x02060300 || PSKC_VERSION_NUMBER >= 0x03000000)
+    {
+      printf ("PSKC_VERSION_NUMBER out of range?!\n");
+      return 1;
+    }
+
   /* Test initialization. */
 
   rc = pskc_global_init ();
