@@ -124,6 +124,48 @@ verbose_totp (oath_totp_flags flags, time_t t0, time_t time_step_size,
 #define generate_otp_p(n) ((n) == 1)
 #define validate_otp_p(n) ((n) == 2)
 
+/* Return newly allocated string with args_info->inputs[i] or, if that
+   string starts with '-' or '@', a line read from stdin (-) or file
+   (@...). */
+const char *
+maybe_read_input (const struct gengetopt_args_info *args_info, int i)
+{
+  const char *given;
+  static FILE *f;
+  static const char *last_file;
+  char *lineptr = NULL;
+  size_t n = 0;
+  ssize_t l;
+
+  if (i >= args_info->inputs_num)
+    return NULL;
+
+  given = args_info->inputs[i];
+
+  if (*given != '-' && *given != '@')
+    return strdup (given);
+
+  if (!strcmp (given, "-"))
+    f = stdin;
+  else if (last_file == NULL || strcmp (given, last_file) != 0)
+    {
+      if (f)
+	fclose (f);
+      f = fopen (given + 1, "r");
+      if (!f)
+	error (EXIT_FAILURE, errno, "open file for KEY/OTP");
+      last_file = given;
+    }
+
+  l = getline (&lineptr, &n, f);
+  if (l < 0)
+    error (EXIT_FAILURE, errno, "read file for KEY/OTP");
+  if (l > 0 && lineptr[--l] == '\n')
+    lineptr[l] = '\0';
+
+  return lineptr;
+}
+
 #define EXIT_OTP_INVALID 2
 
 int
@@ -139,6 +181,7 @@ main (int argc, char *argv[])
   char otp[10];
   time_t now, when, t0, time_step_size;
   int totpflags = 0;
+  const char *inputs[2];
 
   set_program_name (argv[0]);
 
@@ -177,23 +220,25 @@ main (int argc, char *argv[])
     error (EXIT_FAILURE, 0, "liboath initialization failed: %s",
 	   oath_strerror (rc));
 
+  inputs[0] = maybe_read_input (&args_info, 0);
+  inputs[1] = maybe_read_input (&args_info, 1);
+
   if (args_info.base32_flag)
     {
-      rc = oath_base32_decode (args_info.inputs[0],
-			       strlen (args_info.inputs[0]),
-			       &secret, &secretlen);
+      rc = oath_base32_decode (inputs[0],
+			       strlen (inputs[0]), &secret, &secretlen);
       if (rc != OATH_OK)
 	error (EXIT_FAILURE, 0, "base32 decoding failed: %s",
 	       oath_strerror (rc));
     }
   else
     {
-      secretlen = 1 + strlen (args_info.inputs[0]) / 2;
+      secretlen = 1 + strlen (inputs[0]) / 2;
       secret = malloc (secretlen);
       if (!secret)
 	error (EXIT_FAILURE, errno, "malloc");
 
-      rc = oath_hex2bin (args_info.inputs[0], secret, &secretlen);
+      rc = oath_hex2bin (inputs[0], secret, &secretlen);
       if (rc != OATH_OK)
 	error (EXIT_FAILURE, 0, "hex decoding of secret key failed");
     }
@@ -217,12 +262,12 @@ main (int argc, char *argv[])
     error (EXIT_FAILURE, 0, "only digits 6, 7 and 8 are supported");
 
   if (validate_otp_p (args_info.inputs_num) && !args_info.digits_orig)
-    digits = strlen (args_info.inputs[1]);
+    digits = strlen (inputs[1]);
   else if (validate_otp_p (args_info.inputs_num) && args_info.digits_orig &&
-	   args_info.digits_arg != strlen (args_info.inputs[1]))
+	   args_info.digits_arg != strlen (inputs[1]))
     error (EXIT_FAILURE, 0,
 	   "given one-time password has bad length %d != %ld",
-	   args_info.digits_arg, strlen (args_info.inputs[1]));
+	   args_info.digits_arg, strlen (inputs[1]));
 
   if (args_info.inputs_num > 2)
     error (EXIT_FAILURE, 0, "too many parameters");
@@ -249,7 +294,7 @@ main (int argc, char *argv[])
       free (tmp);
 
       if (args_info.inputs_num == 2)
-	printf ("OTP: %s\n", args_info.inputs[1]);
+	printf ("OTP: %s\n", inputs[1]);
       printf ("Digits: %d\n", digits);
       printf ("Window size: %ld\n", window);
     }
@@ -274,7 +319,7 @@ main (int argc, char *argv[])
 	error (EXIT_FAILURE, 0, "cannot parse time `%s'",
 	       args_info.time_step_size_arg);
 
-      for (p = args_info.totp_arg; *p ; p++)
+      for (p = args_info.totp_arg; *p; p++)
 	*p = toupper (*p);
       if (strcmp (args_info.totp_arg, "SHA256") == 0)
 	totpflags = OATH_TOTP_HMAC_SHA256;
@@ -331,12 +376,11 @@ main (int argc, char *argv[])
   else if (validate_otp_p (args_info.inputs_num) && !args_info.totp_given)
     {
       rc = oath_hotp_validate (secret,
-			       secretlen,
-			       moving_factor, window, args_info.inputs[1]);
+			       secretlen, moving_factor, window, inputs[1]);
       if (rc == OATH_INVALID_OTP)
 	error (EXIT_OTP_INVALID, 0,
 	       "password \"%s\" not found in range %ld .. %ld",
-	       args_info.inputs[1],
+	       inputs[1],
 	       (long) moving_factor, (long) moving_factor + window);
       else if (rc < 0)
 	error (EXIT_FAILURE, 0,
@@ -349,13 +393,11 @@ main (int argc, char *argv[])
 				secretlen,
 				when,
 				time_step_size,
-				t0,
-				window,
-				NULL, NULL, totpflags, args_info.inputs[1]);
+				t0, window, NULL, NULL, totpflags, inputs[1]);
       if (rc == OATH_INVALID_OTP)
 	error (EXIT_OTP_INVALID, 0,
 	       "password \"%s\" not found in range %ld .. %ld",
-	       args_info.inputs[1],
+	       inputs[1],
 	       (long) ((when - t0) / time_step_size - window / 2),
 	       (long) ((when - t0) / time_step_size + window / 2));
       else if (rc < 0)
