@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-OATHTOOL=../oathtool
+OATHTOOL=${OATHTOOL:-../oathtool}
 
 ME_=`expr "./$0" : '.*/\(.*\)$'`
 
@@ -40,8 +40,8 @@ dotest()
 	cmp="!="
     fi
 
-    got="$($OATHTOOL $params 2> /dev/null)"
-    err="$($OATHTOOL $params 2>&1 > /dev/null)"
+    got="$(printf "$STDIN" | $OATHTOOL $params 2> /dev/null)"
+    err="$(printf "$STDIN" | $OATHTOOL $params 2>&1 > /dev/null)"
 
     if test "`echo $got`" $cmp "`echo $expect`"; then
 	echo FAIL: oathtool $params
@@ -50,7 +50,11 @@ dotest()
 	echo "stderr:   -$err-"
 	exit 1
     else
-	echo PASS oathtool $params
+	if test -z "$STDIN"; then
+	    echo PASS oathtool $params
+	else
+	    echo PASS oathtool $params STDIN=$(echo $STDIN)
+	fi
     fi
 }
 
@@ -58,6 +62,7 @@ sha1key=3132333435363738393031323334353637383930
 sha256key=3132333435363738393031323334353637383930313233343536373839303132
 sha512key=31323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334
 
+STDIN=""
 dotest "--version" "fail" fail
 dotest "" "fail" fail
 dotest "-h" "fail" fail
@@ -120,6 +125,38 @@ Window size: 0
 Start counter: 0x0 (0)
 
 328482"
+dotest "--verbose 00 328482" "Hex secret: 00
+Base32 secret: AA======
+OTP: 328482
+Digits: 6
+Window size: 0
+Start counter: 0x0 (0)
+
+0"
+dotest " --verbose --totp --now @0 00" "Hex secret: 00
+Base32 secret: AA======
+Digits: 6
+Window size: 0
+TOTP mode: SHA1
+Step size (seconds): 30
+Start time: 1970-01-01 00:00:00 UTC (0)
+Current time: 1970-01-01 00:00:00 UTC (0)
+Counter: 0x0 (0)
+
+328482
+"
+dotest " --verbose --totp=SHA256 --now @0 42" "Hex secret: 42
+Base32 secret: II======
+Digits: 6
+Window size: 0
+TOTP mode: SHA256
+Step size (seconds): 30
+Start time: 1970-01-01 00:00:00 UTC (0)
+Current time: 1970-01-01 00:00:00 UTC (0)
+Counter: 0x0 (0)
+
+654791
+"
 dotest " --verbose --totp=SHA512 --now @0 00" "Hex secret: 00
 Base32 secret: AA======
 Digits: 6
@@ -133,4 +170,26 @@ Counter: 0x0 (0)
 674061
 "
 
+STDIN="gr6d5br725s6vnckv4vlhlaore" dotest "--base32 -" "993210"
+STDIN="gr6d5br725s6vnckv4vlhlaore\n993210\n" dotest "--base32 - -" "0"
+
+tmpfile=`mktemp`
+if ! test -f "$tmpfile"; then
+    tmpfile=tmp
+fi
+tmpfile2=`mktemp`
+if ! test -f "$tmpfile2"; then
+    tmpfile2=tmp2
+fi
+
+trap 'echo rm -f "$tmpfile" "$tmpfile2"' EXIT QUIT
+
+echo gr6d5br725s6vnckv4vlhlaore > "$tmpfile"
+dotest "--base32 @$tmpfile" "993210"
+STDIN=993210 dotest "--base32 @$tmpfile -" "0"
+echo 993210 > "$tmpfile2"
+dotest "--base32 @$tmpfile @$tmpfile2" "0"
+echo 993210 >> "$tmpfile"
+dotest "--base32 @$tmpfile @$tmpfile" "0"
+
 exit 0
