@@ -146,14 +146,16 @@ pam_sm_authenticate (pam_handle_t * pamh,
   char *query_prompt = NULL;
   char *onlypasswd = strdup ("");	/* empty passwords never match */
 
+  /* this has to be first in this function to avoid that cfg contain
+     uninitialized variables. */
+  parse_cfg (flags, argc, argv, &cfg);
+
   if (!onlypasswd)
     {
       retval = PAM_BUF_ERR;
       goto done;
     }
 
-  parse_cfg (flags, argc, argv, &cfg);
-
   retval = pam_get_user (pamh, &user, NULL);
   if (retval != PAM_SUCCESS)
     {
@@ -167,15 +169,16 @@ pam_sm_authenticate (pam_handle_t * pamh,
     time_t last_otp;
     otp[0] = '\0';
     rc = oath_authenticate_usersfile (cfg.usersfile,
-                                      user,
-                                      otp, cfg.window, onlypasswd, &last_otp);
+				      user,
+				      otp, cfg.window, onlypasswd, &last_otp);
 
     DBG (("authenticate first pass rc %d (%s: %s) last otp %s", rc,
-          oath_strerror_name (rc) ? oath_strerror_name (rc) : "UNKNOWN",
-          oath_strerror (rc), ctime (&last_otp)));
+	  oath_strerror_name (rc) ? oath_strerror_name (rc) : "UNKNOWN",
+	  oath_strerror (rc), ctime (&last_otp)));
     if (rc == OATH_UNKNOWN_USER)
       {
-        return PAM_USER_UNKNOWN;
+	retval = PAM_USER_UNKNOWN;
+	goto done;
       }
   }
 
